{% load wagtailcore_tags core_tags i18n %}
<div class="group">
  <ul class="sections footer-menu-list left">
    <li class="footer-menu-list__item">
      <a href="/" class="footer-menu-list__anchor">{% trans "Home" %}</a></li>
    {% load_sections as sections %}
    {% for section in sections %}
      <li class="{{section.get_effective_extra_style_hints}} footer-menu-list__item">
        <a href="{% pageurl section %}" class="footer-menu-list__anchor">
            {{section.title}}
        </a>
      </li>
    {% endfor %}
  </ul>
  <ul class="sections footer-menu-list right">
    {% footer_page %}
    {% if not user.is_authenticated %}
      <li class="footer-menu-list__item">
        <a href="{% url 'molo.profiles:user_register' %}" class="footer-menu-list__anchor">
          {% trans "Join us" %}
        </a>
      </li>
      <li class="footer-menu-list__item">
        <a href="{% url 'molo.profiles:auth_login' %}" class="footer-menu-list__anchor">
          {% trans "Log in" %}
        </a>
      </li>
    {% else %}
      <li class="footer-menu-list__item">
        <a href="{% url 'molo.profiles:view_my_profile' %}" class="footer-menu-list__anchor">
          {% trans "Profile" %}
        </a>
      </li>
      <li class="footer-menu-list__item">
        <a href="{% url 'molo.profiles:auth_logout' %}" class="footer-menu-list__anchor">
          {% trans "Log out" %}
        </a>
      </li>
    {% endif %}
  </ul>
</div>
