.menu-list {
    &--toggle {
      margin: 0;
      @media screen and (min-width:320px) {
        padding: 10px 0;
      }
    }
}

.search {
    &--toggling {
      display: none;
      padding: 15px 0 0 0; //NOTE: This only modify pattern-group inside a component
    }
    &-toggling__button {
      visibility: hidden;
      position: absolute;
      &:not(checked) ~ .search--toggling {
          display: none;
      }
      &:checked ~ .search--toggling {
          display: block;
      }
    }
}
