# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-08-31 09:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0066_add_custom_media_model'),
    ]

    operations = [
        migrations.AddField(
            model_name='molomedia',
            name='feature_in_homepage',
            field=models.BooleanField(default=False),
        ),
    ]
