# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-09-07 06:13
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('profiles', '0016_add_registration_token'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='migrated_username',
            field=models.CharField(blank=True, max_length=40, null=True, validators=[django.core.validators.RegexValidator(b'^[\\w.@+-]+$', 'Enter a valid username. This value may contain only letters, numbers and @/./+/-/_ characters.')], verbose_name='migrated_username'),
        ),
    ]
