{% extends "base.html" %}
{% load i18n %}

{% block content %}
  <div class="heading">
    <h1 class="heading__profiles">{% trans "PIN Changed" %}</h1>
  </div>
  <div class="profiles">
    <h4 class="heading heading--feeback-title">{% trans "Your PIN has been changed sucessfully." %}</h4>
    <a href="{% url 'molo.profiles:auth_login' %}" class="call-to-action__nav-item call-to-action__nav-item--change-password">{% trans "Continue to Sign In" %}</a>
  </div>
{% endblock %}
