/**
Take Note:
Separate smart styles to states folder
*/
.article-teaser {
  overflow: auto;
  &__heading {
      padding: 0 10px;
      text-transform: capitalize;
        &:before {
          content: "#";
        }
  }
}
.base-teaser {
  &__thumbnail-link {
    cursor: pointer;
    outline: none;
    display: inline-block;
    width: 30%;
    vertical-align: top;
    img {
      width: 100%;
    }
  }
  &__title {
    font-size: $font-x-large;
    a {
      color: $charcoal-grey;
    }
  }
  &__title-link {
    color: gray;
  }
  &__subtitle {
    color: $slate-grey;
    font-size: $font-x-small;
    text-transform: none;
  }
}
.teaser-stacklist {
  padding: 0;
  margin: 0;
  &__item {
    display: block;
    padding: 8px;
    position: relative;
    color: #000;
    border-bottom: 4px solid #edf0f2;
    font-size: 0;
    .comment-counter {
      position: absolute;
      right: 10px;
      bottom: 3px;
      display: inline-block;
      height: 25px;
      &__numeral {
        color: $slate-grey;
        font-size: $font-xx-small;
        vertical-align: bottom;
        margin-left: 5px;
      }
    }
  }
  &__item--content {
    display: inline-block;
    width: 60%;
    vertical-align: top;
    padding: 0 0 0 10px;
  }
}
