# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-15 13:34
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailimages', '0021_image_file_hash'),
        ('core', '0008_create_forms_index_pages'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='telegram_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='telegram_sharing',
            field=models.BooleanField(default=False, help_text='Enable this field to allow for sharing to Telegram.', verbose_name='Telegram'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='whatsapp_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='whatsapp_sharing',
            field=models.BooleanField(default=False, help_text='Enable this field to allow for sharing to Whatsapp.', verbose_name='Whatsapp'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='youtube_image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='youtube_sharing',
            field=models.BooleanField(default=False, help_text='Enable this field to allow for sharing to YouTube.', verbose_name='YouTube'),
        ),
    ]
