# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-05-21 10:39
from __future__ import unicode_literals

from django.db import migrations, models
import django_enumfield.db.fields
import molo.core.models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0014_sitesettings_article_ordering_within_section'),
    ]

    operations = [
        migrations.AddField(
            model_name='bannerpage',
            name='hide_banner_on_freebasics',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='article_ordering_within_section',
            field=django_enumfield.db.fields.EnumField(blank=True, default=None, enum=molo.core.models.ArticleOrderingChoices, help_text='Ordering of articles within a section', null=True),
        ),
    ]
